

; Function to correct MOD09 data following the method proposed in Wang et al. (2016),
; A simple correction method for the MODIS surface reflectance product over typical inland waters in China. 
; International Journal of Remote Sensing, 37(24).

FUNCTION MOD09_CORRECT,LAYER_DATA,n_s,n_l,n_b

  medlayers=intarr(n_s,n_l,4)  ;
  minlayer=intarr(n_s,n_l)
  layers=intarr(n_s,n_l,n_b)
  ;ignore negative values
  medlayers[*,*,0]=layer_data[*,*,1]*(layer_data[*,*,1] ge 0)
  medlayers[*,*,1]=layer_data[*,*,4]*(layer_data[*,*,4] ge 0)
  medlayers[*,*,2]=layer_data[*,*,5]*(layer_data[*,*,5] ge 0)
  medlayers[*,*,3]=layer_data[*,*,6]*(layer_data[*,*,6] ge 0)
  ;layers=0b
  ;minimum reflectance in NIR-SWIR bands
  minlayer=medlayers[*,*,0]
  minlayer=minlayer*(minlayer lt medlayers[*,*,1])+medlayers[*,*,1]*(medlayers[*,*,1] le minlayer )
  minlayer=minlayer*(minlayer lt medlayers[*,*,2])+medlayers[*,*,2]*(medlayers[*,*,2] le minlayer )
  minlayer=minlayer*(minlayer lt medlayers[*,*,3])+medlayers[*,*,3]*(medlayers[*,*,3] le minlayer )
  medlayers=0b
  ; 
  ;minimum reflectance in RGB
  minRGB=intarr(n_s,n_l)
  minRGB=layer_data[*,*,0]*(layer_data[*,*,0] lt layer_data[*,*,3])+layer_data[*,*,3]*(layer_data[*,*,0] ge layer_data[*,*,3])
  minRGB=minRGB*(minRGB lt layer_data[*,*,2])+layer_data[*,*,2]*(minRGB ge layer_data[*,*,2])

  ;Correction
  for j = 0, n_b-1,1 do begin

    layers[*,*,j] = (layer_data[*,*,j]-minlayer)*(minlayer le minRGB)+layer_data[*,*,j]*(minlayer GT minRGB)

  endfor

  ;avoid negative reflectance in the red band
  for j = 1, n_b-1,1 do begin
    layers[*,*,j] = layers[*,*,j]*(layers[*,*,0] ge 0)+(layers[*,*,j]-layers[*,*,0])*(layers[*,*,0] lt 0)
  endfor
  layers[*,*,0]=layers[*,*,0]*(layers[*,*,0] ge 0)

  minlayer=0b
  layer_data=0b
  return,layers
END
;====================================================================================================================================
;
;Function to calculate FUI
FUNCTION Angle_FUI_LUT ,arctan,n_s,n_l
  index=intarr(n_s,n_l)
  Angle_FUI=fltarr(23)
  Angle_FUI=[38.0,40.46700462,45.19625958,52.85273477,67.16945342,91.29804253,122.5851964,151.4792393,170.4629157,181.4982872,191.8352115,199.0383224,205.0622424,210.5766446,216.5569027,222.1153011,227.6293157,232.8302318,237.352272,241.7591989,245.5513068,248.9529246,252.0]
  for m = 0,n_s-1 do begin
    for n= 0,n_l-1 do begin
      near = MIN(ABS(Angle_FUI-arctan[m,n]), indice)
      index[m,n]=indice
    endfor
  endfor
  RETURN ,index
END

;
;==============================================================================================================================
;
;Main program
;To calculate FUI using MOD09 data
;Input data: MOD09 images with 7 bands in HDF format
;Output data: FUI images (GeoTiff format)
;BY SHENGLEI Wang
;shengleiw@126.com

pro MODIS_FUI,ev
  COMPILE_OPT idl2

;Step 1: input MODIS image file
  nFiles=0 ;Init the number of MERIS files

  inputFiles = envi_pickfile(title = 'Please choose the input MERIS images', filter ='*.hdf', /multiple_files)
  nFiles = N_ELEMENTS(inputFiles);Get the number of input files
  if inputFiles[0] eq '' then return

;Step 2: Calculate the FUI and hue angle for the MERIS images

;Set progress bar
  statBase = WIDGET_AUTO_BASE(title='Calculating the FUI...')
  statText = ['Run FUI calculation' ];
  envi_report_init, statText, title='FUI calculation', base=statBase, /interrupt;Init the progress bar


  FOR a=0,nFiles-1 DO BEGIN

    ns=0     ;
    nl=0     ;
    nb=0     ;
    envi_report_stat, statBase, a+1, nFiles;

    ;open data
    envi_open_file, inputFiles[a], r_fid=fid

    if fid ne -1 then begin
      envi_file_query, fid, ns=ns, nl=nl, nb=nb
      Dims=[-1, 0, ns-1, 0, nl-1]  
      map_info = envi_get_map_info(fid=fid) ;
      layers= intarr(ns,nl,nb) ;
      nlayers=fltarr(ns,nl,3)  ;
      zlayers=intarr(ns,nl,2)  ;

      ;read data
      for i = 0, nb-1,1 do begin
        layers_ini= fltarr(ns,nl,nb) ;
        layers_ini[*,*,i] = envi_get_data(fid=fid,dims=dims,pos=i)
        layers[*,*,i] = layers_ini[*,*,i]*10000
        layers_ini=0b  ;
      endfor
      ;MOD09 correction to derive water-leaving reflectance
      layers=MOD09_CORRECT(LAYERS,NS,NL,NB)

      ;XYZ calculation
      nlayers[*,*,0]=2.7689*layers[*,*,0]+1.7517*layers[*,*,3]+1.1302*layers[*,*,2]
      nlayers[*,*,1]=1.0000*layers[*,*,0]+4.5907*layers[*,*,3]+0.0601*layers[*,*,2]
      nlayers[*,*,2]=0.0565*layers[*,*,3]+5.5943*layers[*,*,2]

      ;xy calculation
      zlayers[*,*,0]=10000*nlayers[*,*,0]/(nlayers[*,*,0]+nlayers[*,*,1]+nlayers[*,*,2])
      zlayers[*,*,1]=10000*nlayers[*,*,1]/(nlayers[*,*,0]+nlayers[*,*,1]+nlayers[*,*,2])

      zlayers[*,*,0]=zlayers[*,*,0]*(zlayers[*,*,0] ge 0)
      zlayers[*,*,0]=zlayers[*,*,0]*(zlayers[*,*,0] le 10000)

      zlayers[*,*,1]=zlayers[*,*,1]*(zlayers[*,*,1] ge 0)
      zlayers[*,*,1]=zlayers[*,*,1]*(zlayers[*,*,1] le 10000)
      
      nlayers=0b
      layers=0b
      ENVI_FILE_MNG, id=fid, /REMOVE

      atanxy=fltarr(ns,nl) ;
      colorps=intarr(ns,nl,3) ;

      atanxy=atan(zlayers[*,*,0]-3333, zlayers[*,*,1]-3333)*180/!pi+180 ;this atanxy is not the hue angle in Woerd (2015), but that in Wang et al. (2018), atanxy =(270 - hue angle).
      ;Delta correction
      atanxy=atanxy+(-1.8185*(atanxy/100)^5-62.461*(atanxy/100)^4+320.49*(atanxy/100)^3-510.97*(atanxy/100)^2+306.32*atanxy/100-50.343)
      zlayers=0b
    ; Calculate FUI using a look-up table function
      FUI=BYTARR(NS,NL)
      FUI=Angle_FUI_LUT(atanxy,ns,nl)
      atanxy=0b
      bnames=['FUI']
      nlen=strlen(inputFiles[a])
      FUI_file = strmid(inputFiles[a],0,nlen-4)+'_FUI'; output file name
      envi_write_envi_file,FUI,out_name=FUI_file,map_info=map_info,bnames=bnames,/no_open;output data
      FUI=0B
      nlen=0b
    endif  else begin;if fid
      print,'cannot open '+inputFile_MOD09[a]
    endelse
  ENDFOR

  envi_report_init, base=statBase, /finish;

end